/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
package net.smartlab.web.auth;

import net.smartlab.web.BusinessObject;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;

/**
 * This class represents a
 * 
 * @author svetrini
 * @author rlogiacco
 * @hibernate.class schema="auth" table="`permission`"
 */
public final class Permission extends BusinessObject implements Cloneable {

	private static final long serialVersionUID = -4074180118048290779L;

	/**
	 * @uml.property name="id"
	 */
	private long id;

	/**
	 * The role to which this permission is granted.
	 * 
	 * @directed true
	 * @uml.property name="roleId"
	 */
	private String roleId;

	/**
	 * The subject to which this permission is granted.
	 * 
	 * @directed true
	 * @uml.property name="subject"
	 * @uml.associationEnd multiplicity="(1 1)"
	 *                     inverse="permissions:net.smartlab.web.auth.Subject"
	 */
	private Subject subject;

	/**
	 * The scope onto which this permission is granted.
	 * 
	 * @directed true
	 * @uml.property name="scope"
	 * @uml.associationEnd inverse="permission:net.smartlab.web.auth.Scope"
	 */
	private Scope scope;

	/**
	 * The handler class which can further validate the permission.
	 * 
	 * @directed true
	 * @uml.property name="handler"
	 * @uml.associationEnd 
	 *                     inverse="permission:net.smartlab.web.auth.AuthorizationHandler"
	 */
	private AuthorizationHandler handler;


	/**
	 * Default constructor.
	 */
	public Permission() {
		super();
	}

	/**
	 * Returns the id.
	 * 
	 * @return the id.
	 * @hibernate.id column="`id`" unsaved-value="0" generator-class="native"
	 * @uml.property name="id"
	 */
	public long getId() {
		return id;
	}

	/**
	 * Sets the id.
	 * 
	 * @param id the id to set.
	 * @uml.property name="id"
	 */
	protected void setId(long id) {
		this.id = id;
	}

	/**
	 * Returns the subject to which this permission is granted.
	 * 
	 * @return the subject to which this permission is granted.
	 * @hibernate.many-to-one column="`subject`" update="false" inverse="true"
	 * @uml.property name="subject"
	 */
	public Subject getSubject() {
		return subject;
	}

	/**
	 * Sets the subject to which this permission is granted.
	 * 
	 * @param subject the principal to which this permission is granted.
	 * @uml.property name="subject"
	 */
	public void setSubject(Subject subject) {
		this.subject = subject;
	}

	/**
	 * Returns the scope onto which this permission is granted.
	 * 
	 * @return the scope onto which this permission is granted.
	 * @hibernate.component
	 * @uml.property name="scope"
	 */
	public Scope getScope() {
		return scope;
	}

	/**
	 * Sets the scope onto which this permission is granted.
	 * 
	 * @param scope the scope onto which this permission is granted.
	 * @uml.property name="scope"
	 */
	public void setScope(Scope scope) {
		this.scope = scope;
	}

	/**
	 * Returns the <code>AuthorizationHandler</code> reference name used for
	 * extended authorization controls.
	 * 
	 * @return the <code>AuthorizationHandler</code> reference name.
	 * @hibernate.property column="`handler`"
	 * @uml.property name="handler"
	 */
	public String getHandler() {
		return (handler != null ? handler.getId() : null);
	}

	/**
	 * Sets the <code>AuthorizationHandler</code> reference name used for
	 * extended authorization controls.
	 * 
	 * @param handler the <code>AuthorizationHandler</code> reference name to
	 *        set.
	 * @uml.property name="handler"
	 */
	public void setHandler(String handler) {
		this.handler = Domain.getInstance().getAuthorizationHandler(handler);
	}

	/**
	 * TODO documentation
	 * 
	 * @return
	 * @uml.property name="role"
	 */
	public Role getRole() {
		return (Role)RoleFactory.getInstance().findByKey(roleId);
	}

	/**
	 * Returns the role to which this permission is granted.
	 * 
	 * @return the role to which this permission is granted.
	 * @hibernate.property column="`roleId`"
	 * @uml.property name="roleId"
	 */
	public String getRoleId() {
		return this.roleId;
	}

	/**
	 * Sets the role to which this permission is granted.
	 * 
	 * @param role the role to which this permission is granted.
	 * @uml.property name="roleId"
	 */
	public void setRoleId(String role) {
		this.roleId = role;
	}

	/**
	 * @see net.smartlab.web.BusinessObject#getVersion()
	 * @hibernate.version
	 */
	public long getVersion() {
		return super.getVersion();
	}

	/**
	 * @see java.lang.Object#equals(Object)
	 */
	public boolean equals(Object object) {
		if (!(object instanceof Permission)) {
			return false;
		}
		return new EqualsBuilder().append(this.roleId,((Permission)object).roleId)
				.append(this.subject, ((Permission)object).subject)
				.append(this.scope, ((Permission)object).scope).isEquals();
	}

	/**
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return new HashCodeBuilder(1683017233, 1682367525).append(this.roleId).append(this.subject).append(this.scope)
				.toHashCode();
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return new ToStringBuilder(this).append("roleId", this.roleId).append("scope", this.scope).toString();
	}
	
	/**
	 * @see java.lang.Object#clone()
	 */
	protected Object clone() throws CloneNotSupportedException {
		try {
			return BeanUtils.cloneBean(this);
		} catch (Exception e) {
			logger.error(e);
			return null;
		}
	}
}
