/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
package net.smartlab.web.auth;

import java.util.StringTokenizer;

import javax.servlet.jsp.JspException;

import net.smartlab.web.BusinessException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.taglib.TagUtils;

/**
 * This JSP tag is used to check the validity of the current user both in terms
 * of authentication and authorization. In its basic usage the tag checks if a
 * user is associated to the current session. If the optional <code>role</code>
 * attribute is specified it identifies a comma separated list of roles to which
 * the user <b>may </b> be associated to pass the validity check. In addition a
 * context could be optionally specified using the <code>name</code> attribute
 * causing the validity check to be performed on the permessions allowed on the
 * <code>SecureBusinessObject</code> binded with that name.
 * 
 * @author rlogiacco
 * @author svetrini
 * @jsp.tag name = "isValid" description="checks if a user was previously
 *          authenticated or if it's authorized by a role" body-content="JSP"
 */
public class ValidTag extends org.apache.struts.taglib.logic.PresentTag {

	/**
	 * Provides logging for the test cases.
	 * 
	 * @uml.property name="logger"
	 * @uml.associationEnd multiplicity="(1 1)"
	 */
	protected Log logger = LogFactory.getLog(this.getClass());

	private static final long serialVersionUID = -3426773639709918540L;

	/**
	 * The roles the user <b>may</b> be associated to pass the validity check.
	 * 
	 * @uml.property name="roles"
	 * @uml.associationEnd multiplicity="(0 -1)"
	 */
	private Role[] roles;

	/**
	 * @uml.property name="functions"
	 * @uml.associationEnd multiplicity="(0 -1)"
	 */
	private Privilege[] privileges;

	/**
	 * @uml.property name="scope"
	 */
	private String scope;

	/**
	 * @uml.property name="inherit"
	 */
	private boolean inherit = true;


	/**
	 * Default empty constructor.
	 */
	public ValidTag() {
		super.name = UserAction.SESSION_KEY;
		super.scope = "session";
	}

	/**
	 * @see org.apache.struts.taglib.logic.ConditionalTagBase#condition()
	 */
	protected boolean condition() throws JspException {
		return (condition(true));
	}

	/**
	 * @see org.apache.struts.taglib.logic.PresentTag#condition(boolean)
	 */
	public boolean condition(boolean desired) throws JspException {
		User user = UserAction.getUser(super.pageContext.getRequest());
		if (User.isGuest(user)) {
			return !desired;
		}
		if (roles != null || privileges != null) {
			// a user was authenticated and roles or privileges were specified
			try {
				Scope scope = null;
				if (this.scope != null) {
					// a scope has been specified
					String name = this.scope;
					String property = null;
//					if (name.indexOf('.') > -1) {
//						property = name.substring(name.indexOf('.') + 1);
//						name = name.substring(0, name.indexOf('.'));
//					}
					Object object = TagUtils.getInstance().lookup(pageContext, name, property, null);
					Scope.Type type = Domain.getInstance().findScopeType(object.getClass());
					if (type != null) {
						scope = new Scope();
						scope.setTypeId(type.getId());
						scope.setKey(Scope.getKey(object));
					}
				}
				if (roles != null) {
					// a set of roles has been specified
					for (int i = 0; i < roles.length; i++) {
						boolean isInRole = user.isInRole(roles[i], scope, inherit);
						if (isInRole) {
							return desired;
						}
					}
				}
				if (privileges != null) {
					// a set of privileges has been specified
					for (int i = 0; i < privileges.length; i++) {
						boolean hasPrivilege = user.hasPrivilege(privileges[i], scope, null);
						if (!hasPrivilege) {
							return !desired;
						}
					}
					return desired;
				}
				return !desired;
			} catch (BusinessException be) {
				throw new JspException(be);
			}
		}
		return desired;
	}

	/**
	 * @jsp.attribute required="false" rtexprvalue="true" type="string"
	 *                description="a comma separated list of authorization
	 *                roles"
	 */
	public void setRole(String role) {
		StringTokenizer tokenizer = new StringTokenizer(role, ",");
		this.roles = new Role[tokenizer.countTokens()];
		for (int i = 0; i < roles.length; i++) {
			roles[i] = new Role(tokenizer.nextToken().trim());
		}
	}
	
	/**
	 * @see #setPrivilege(java.lang.String)
	 * @deprecated use setPrivilege(java.lang.String) instead
	 */
	public void setFunction(String privilege) {
		this.setPrivilege(privilege);
	}

	/**
	 * @jsp.attribute required="false" rtexprvalue="true" type="string"
	 *                description="a comma separated list of privileges"
	 */
	public void setPrivilege(String privilege) {
		StringTokenizer tokenizer = new StringTokenizer(privilege, ",");
		this.privileges = new Privilege[tokenizer.countTokens()];
		for (int i = 0; i < privileges.length; i++) {
			String token = tokenizer.nextToken().trim();
			int separator = token.indexOf('#');
			if (separator > -1) {
				privileges[i] = new Privilege(token.substring(0, separator), token.substring(separator + 1), (byte)0x00);
			} else {
				privileges[i] = new Privilege(token, Privilege.ANY, (byte)0x00);
			}
		}
	}

	/**
	 * @jsp.attribute required="false" rtexprvalue="true" type="string"
	 *                description="the name of the attribute representing the
	 *                context against which the authorization must be performed"
	 */
	public void setScope(String scope) {
		this.scope = scope;
	}

	/**
	 * @jsp.attribute required="false" rtexprvalue="true" type="boolean"
	 *                description="specifies if inherited permissions can be considered for matching checks (defaults to true)"
	 */
	public void setInherit(boolean inherit) {
		this.inherit = inherit;
	}
}
